using System;
using System.Configuration;
using System.Net;
using System.Reflection;
using System.Security.Cryptography.X509Certificates;
using System.ServiceModel.Configuration;
using System.Windows.Forms;

namespace ServiceTester
{
    public partial class Principal : Form
    {
        private static bool _valida;

        public Principal()
        {
            _valida = true;
            InitializeComponent();
            System.Net.ServicePointManager.ServerCertificateValidationCallback = ((sender, certificate, chain, sslPolicyErrors) => true);
        }

        private void Principal_Load(object sender, EventArgs e)
        {
            ServicePointManager.ServerCertificateValidationCallback = new System.Net.Security.RemoteCertificateValidationCallback(OnValidationCallback);
            CarregarUser();
            CarregarListaServicos();
        }

        private void CarregarUser()
        {
            if (!string.IsNullOrEmpty(ConfigurationManager.AppSettings["Usuario"]))
            {
                txtUser.Text = ConfigurationManager.AppSettings["Usuario"];
            }

            if (!string.IsNullOrEmpty(ConfigurationManager.AppSettings["Password"]))
            {
                txtPwd.Text = ConfigurationManager.AppSettings["Password"];
            }
        }

        private void CarregarListaServicos()
        {
            TreeNode root = tvwServicos.Nodes[0];
            TreeNode nodeWCF = root.Nodes[0];

            root.Text = "Aguarde... Carregando Lista de Servicos";

            ClientSection clientSection;
            clientSection = (ConfigurationManager.GetSection("system.serviceModel/client") as ClientSection);

            foreach (ChannelEndpointElement endpoint in clientSection.Endpoints)
            {
                string contract = endpoint.Contract;
                if (endpoint.Contract.Contains("IWCFIntegracao"))
                    contract = "ServiceTester." + endpoint.Contract;
                
                Type type = Type.GetType(contract);
                if (type != null)
                {
                    if (!endpoint.Name.Contains("IGerenciadorArquivo"))
                    {
                        TreeNode serviceNode = new TreeNode(endpoint.Name.Substring(endpoint.Name.IndexOf("_I")).Replace("_I", string.Empty));
                        serviceNode.Tag = type;
                        foreach (MethodInfo method in type.GetMethods())
                        {
                            if (!serviceNode.Nodes.ContainsKey(method.Name) && !method.IsSpecialName)
                            {
                                if ((!method.Name.Equals("ConsultarQtdeTotalProdutos") && !method.Name.Contains("Async")) && (!method.Name.Contains("Foto")) && (!method.Name.Contains("ConsultarEditalEnviado")) && (!method.Name.Contains("ConsultarEditalResultado")))
                                {
                                    TreeNode methodNode = new TreeNode(method.Name);
                                    methodNode.Tag = method;
                                    serviceNode.Nodes.Add(methodNode);
                                }
                            }
                        }
                        nodeWCF.Nodes.Add(serviceNode);
                    }
                }
            }            
            root.Text = "Servicos Disponveis";
            root.ExpandAll();
        }

        private void tvwServicos_NodeMouseDoubleClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            this.Cursor = System.Windows.Forms.Cursors.WaitCursor;
            if (string.IsNullOrEmpty(txtUser.Text) || string.IsNullOrEmpty(txtPwd.Text))
            {
                MessageBox.Show(" necessrio informar usurio e senha");
                if (string.IsNullOrEmpty(txtUser.Text))
                {
                    txtUser.Focus();
                }
                else if (string.IsNullOrEmpty(txtPwd.Text))
                {
                    txtPwd.Focus();
                }
                this.Cursor = System.Windows.Forms.Cursors.Arrow;
            }
            else
            {
                ConfigurationManager.AppSettings["Usuario"] = txtUser.Text;
                ConfigurationManager.AppSettings["Password"] = txtPwd.Text;
                if (e.Node.Nodes.Count == 0)
                {
                    Type type = (e.Node.Parent.Tag as Type);
                    if (type != null)
                    {
                        MethodInfo method = (e.Node.Tag as MethodInfo);
                        if (method != null)
                        {
                            try
                            {
                                Form _form;
                                if (e.Node.Text.Equals("ConsultarEdital"))
                                {
                                    _form = new Edital.ConsultarEditalResultado();
                                    _form.Text = method.Name;
                                }
                                else if (e.Node.Text.Contains("ConsultarProduto"))
                                {
                                    _form = new Produto.ConsultarProduto();
                                    _form.Text = method.Name;
                                }
                                else if (e.Node.Parent.FullPath.Contains("WCFIntegracao"))
                                {
                                    if (e.Node.Text.Equals("ConsultarProduto"))
                                    {
                                        _form = new Produto.ConsultarProduto();
                                        _form.Text = method.Name;
                                    }
                                    else if (e.Node.Text.Equals("ConsultarUnidadesMedida"))
                                    {
                                        _form = new Produto.ConsultarUnidadesMedida();
                                        _form.Text = method.Name;
                                    }
                                    else
                                    {
                                        _form = new Testers.WCFIntegracao.FormWCFIntegracao(method);
                                        _form.Text = "WS Integracao - " + method.Name;
                                    }
                                }
                                else
                                {
                                    Type formType = Type.GetType(type.Namespace + "." + method.Name, false, true);
                                    _form = (Form)Assembly.GetAssembly(formType).CreateInstance(formType.FullName);
                                    _form.Text = method.Name;
                                }

                                _form.Show(this);
                                this.Cursor = System.Windows.Forms.Cursors.Arrow;
                            }
                            catch (Exception ex)
                            {
                                this.Cursor = System.Windows.Forms.Cursors.Arrow;
                                MessageBox.Show("No foi possvel construir uma instncia do aplicativo de teste. \n" + ex.Message,
                                                "Configurao incorreta.",
                                                MessageBoxButtons.OK,
                                                MessageBoxIcon.Error);
                            }
                        }
                    }
                    else
                    {
                        Form _form;
                        Type formType = Type.GetType("ServiceTester.Edital.ConsultarEditalResultado", false, true);
                        _form = (Form)Assembly.GetAssembly(formType).CreateInstance(formType.FullName);
                        _form.Text = "ConsultarEdital";
                        _form.Show(this);
                        this.Cursor = System.Windows.Forms.Cursors.Arrow;
                    }
                }
            }
        }

        private void chkValidaCert_Click(object sender, EventArgs e)
        {
            _valida = !chkValidaCert.Checked;
        }

        static bool OnValidationCallback(object sender, X509Certificate cert, X509Chain chain, System.Net.Security.SslPolicyErrors errors)
        {
            if (!_valida || errors.Equals(System.Net.Security.SslPolicyErrors.None))
            {
                return true;
            }

            MessageBox.Show("Ocorreu o seguinte erro de validao de certificado:\n\n" + errors.ToString(), "Validao de Certificado");
            return false;
        }
    }
}