﻿using ServiceTester.Integracao;
using ServiceTester.Util;
using System;
using System.Configuration;
using System.IO;
using System.Reflection;
using System.ServiceModel;
using System.Text;
using System.Windows.Forms;

namespace ServiceTester.Testers.WCFIntegracao
{
    public partial class FormWCFIntegracao : Form
    {
        private int tabResult = 4;
        public FormWCFIntegracao(MethodInfo method)
        {
            InitializeComponent();
            switch (method.Name)
            {
                case "SalvarProcessamento":
                    SetUpTabControl(0);
                    break;
                case "SalvarAnexoProcessamento":
                    SetUpTabControl(1);
                    break;
                case "SalvarFotoProcessamento":
                    SetUpTabControl(2);
                    break;
                case "ConsultarSituacaoProcessamento":
                    SetUpTabControl(3);
                    break;
                default:
                    break;
            }

            tabControl1.TabPages.Remove(tabPage3);

            System.Net.ServicePointManager.ServerCertificateValidationCallback = ((sender, certificate, chain, sslPolicyErrors) => true);
        }
        private void FormWCFIntegracao_Load(object sender, EventArgs e)
        {
            string login = ConfigurationManager.AppSettings["usuario"];
            txtUser.Enabled = false;
            txtUser.Text = login;
        }
        private void SetUpTabControl(int tab)
        {
            tabControl1.SelectedTab = tabControl1.TabPages[tab];
        }
        private string GetFileName(string filePath)
        {
            FileInfo fi = new FileInfo(filePath);
            return fi.Name;
        }
        private byte[] GetFileByteArray(string filePath)
        {
            FileStream fs = new FileStream(filePath, FileMode.Open, FileAccess.Read);
            byte[] retorno = new byte[fs.Length];

            fs.Read(retorno, 0, System.Convert.ToInt32(fs.Length));
            fs.Close();

            return retorno;
        }
        private StringBuilder erroMsg = null;
        private void btProc_Click(object sender, EventArgs e)
        {
            string result = string.Empty;
            this.Cursor = System.Windows.Forms.Cursors.WaitCursor;
            try
            {
                erroMsg = new StringBuilder();
                if (ValidarDados())
                {
                    using (WCFIntegracaoClient proxy = ServiceFactory.GetIntegracaoInstance())
                    {
                        IWCFIntegracao channel = proxy.ChannelFactory.CreateChannel();
                        result = channel.EnviarEdital(txtXml.Text);
                        txtResult.Text = result;
                        tabControl1.SelectedTab = tabControl1.TabPages[tabControl1.TabPages.Count - 1];
                    }
                }
                else
                {
                    MessageBox.Show(erroMsg.ToString());
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            finally
            {
                this.Cursor = System.Windows.Forms.Cursors.Arrow;
            }
        }
        private bool ValidarDados()
        {
            if (string.IsNullOrEmpty(txtXml.Text))
            {
                erroMsg.AppendLine("É obrigatória seleção de uma arquivo xml.");
                return false;
            }

            return true;
        }
        private void btAnexo_Click(object sender, EventArgs e)
        {
            string result = "Esse serviço não retorna dados. Para saber o status do Edital, execute o método \"Consultar Edital Enviado.\"";
            this.Cursor = System.Windows.Forms.Cursors.WaitCursor;
            try
            {
                using (WCFIntegracaoClient proxy = ServiceFactory.GetIntegracaoInstance())
                {
                    IWCFIntegracao channel = proxy.ChannelFactory.CreateChannel();
                    byte[] file = GetFileByteArray(txtArquivoAnexo.Text);
                    string fileName = GetFileName(txtArquivoAnexo.Text);
                    channel.EnviarAnexo(txtProtocoloAnexo.Text, Convert.ToInt32(txtSequenciaAnexo.Text), file, fileName, fileName, true);
                    txtResult.Text = result;
                    tabControl1.SelectedTab = tabControl1.TabPages[tabControl1.TabPages.Count - 1];
                }
            }
            catch (FaultException ex)
            {
                MessageBox.Show(ex.Message);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            finally
            {
                this.Cursor = System.Windows.Forms.Cursors.Arrow;
            }
        }
        private void btFoto_Click(object sender, EventArgs e)
        {
            string result = "Esse serviço não retorna dados. Para saber o status do Edital, execute o método \"Consultar Edital Enviado.\"";
            this.Cursor = System.Windows.Forms.Cursors.WaitCursor;
            try
            {
                using (WCFIntegracaoClient proxy = ServiceFactory.GetIntegracaoInstance())
                {
                    IWCFIntegracao channel = proxy.ChannelFactory.CreateChannel();
                    byte[] file = GetFileByteArray(txtArquivoFoto.Text);
                    string fileName = GetFileName(txtArquivoFoto.Text);
                    channel.EnviarFoto(txtProtocoloFoto.Text,
                                                    Convert.ToDecimal(txtProdBBM.Text),
                                                    txtProdOrgao.Text,
                                                    Convert.ToInt32(txtLote.Text),
                                                    file,
                                                    fileName,
                                                    Convert.ToInt32(txtSequenciaFoto.Text),
                                                    Convert.ToInt32(txtItem.Text));
                    txtResult.Text = result;
                    tabControl1.SelectedTab = tabControl1.TabPages[tabControl1.TabPages.Count - 1];
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            finally
            {
                this.Cursor = System.Windows.Forms.Cursors.Arrow;
            }
        }
        private void btConsulta_Click(object sender, EventArgs e)
        {
            string result = string.Empty;
            this.Cursor = System.Windows.Forms.Cursors.WaitCursor;
            try
            {
                using (WCFIntegracaoClient proxy = ServiceFactory.GetIntegracaoInstance())
                {
                    IWCFIntegracao channel = proxy.ChannelFactory.CreateChannel();

                    result = channel.ConsultarEditalEnviado(txtProtocoloConsulta.Text);
                    txtResult.Text = result;
                    tabControl1.SelectedTab = tabControl1.TabPages[tabControl1.TabPages.Count - 1];
                }
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            finally
            {
                this.Cursor = System.Windows.Forms.Cursors.Arrow;
            }
        }
        private void button1_Click(object sender, EventArgs e)
        {
            openFileDialog1.ShowDialog();
            try
            {
                txtXmlFile.Text = openFileDialog1.FileName;
                StreamReader sr = new StreamReader(txtXmlFile.Text);
                txtXml.Text = sr.ReadToEnd();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }
        private void button2_Click(object sender, EventArgs e)
        {
            openFileDialog1.ShowDialog();
            txtArquivoAnexo.Text = openFileDialog1.FileName;
        }
        private void button3_Click(object sender, EventArgs e)
        {
            openFileDialog1.ShowDialog();
            txtArquivoFoto.Text = openFileDialog1.FileName;
        }
    }
}