﻿using System;
using System.Collections.ObjectModel;
using System.ServiceModel;
using System.ServiceModel.Channels;
using System.ServiceModel.Description;
using System.ServiceModel.Dispatcher;

namespace ServiceTester.Util
{
    [AttributeUsage(AttributeTargets.Class)]
    public class AuthenticationInspectorBehavior : Attribute, IDispatchMessageInspector, IEndpointBehavior, IClientMessageInspector, IServiceBehavior
    {
        #region IDispatchMessageInspector

        public const string C_HEADER_KEY = "C_HEADER_KEY";
        public const string C_HEADER_NS = "C_HEADER_NS";

        public object AfterReceiveRequest(ref Message request, IClientChannel channel, InstanceContext instanceContext)
        {
            throw new FaultException("Esta DLL deverá ser utilizada apenas no cliente");
        }

        public void BeforeSendReply(ref Message reply, object correlationState)
        {
        }

        #endregion

        #region IEndpointBehavior

        public void Validate(ServiceEndpoint endpoint)
        {
        }

        public void AddBindingParameters(ServiceEndpoint endpoint, BindingParameterCollection bindingParameters)
        {
        }

        public void ApplyDispatchBehavior(ServiceEndpoint endpoint, EndpointDispatcher endpointDispatcher)
        {
            var channelDispatcher = endpointDispatcher.ChannelDispatcher;
            if (channelDispatcher == null) return;
            foreach (var ed in channelDispatcher.Endpoints)
            {
                var inspector = new AuthenticationInspectorBehavior();
                ed.DispatchRuntime.MessageInspectors.Add(inspector);
            }
        }

        public void ApplyClientBehavior(ServiceEndpoint endpoint, ClientRuntime clientRuntime)
        {
            var inspector = new AuthenticationInspectorBehavior();
            clientRuntime.MessageInspectors.Add(inspector);
        }

        #endregion

        #region IClientMessageInspector

        public object BeforeSendRequest(ref Message request, IClientChannel channel)
        {

            var webUser = new MessageHeader<string>(AppConfiguration.GetUser());
            var webpassword = new MessageHeader<string>(AppConfiguration.GetPassword());

            var webUserHeader = webUser.GetUntypedHeader("web-user", "ns");
            var webPasswordHeader = webpassword.GetUntypedHeader("web-password", "ns");

            request.Headers.Add(webUserHeader);
            request.Headers.Add(webPasswordHeader);

            return null;
        }

        public void AfterReceiveReply(ref Message reply, object correlationState)
        {
        }

        #endregion

        #region IServiceBehaviour

        public void Validate(ServiceDescription serviceDescription, ServiceHostBase serviceHostBase)
        {
        }

        public void AddBindingParameters(ServiceDescription serviceDescription, ServiceHostBase serviceHostBase, Collection<ServiceEndpoint> endpoints, BindingParameterCollection bindingParameters)
        {
        }

        public void ApplyDispatchBehavior(ServiceDescription serviceDescription, ServiceHostBase serviceHostBase)
        {
            foreach (ChannelDispatcher cDispatcher in serviceHostBase.ChannelDispatchers)
            {
                foreach (var eDispatcher in cDispatcher.Endpoints)
                {
                    eDispatcher.DispatchRuntime.MessageInspectors.Add(new AuthenticationInspectorBehavior());
                }
            }
        }

        #endregion
    }
}
